module Foundation.Monad.Base
    ( Functor(..)
    , Applicative(..)
    , Monad(..)
    , MonadIO(..)
    , MonadFailure(..)
    , MonadThrow(..)
    , MonadCatch(..)
    , MonadTrans(..)
    , IdentityT
    ) where

import Basement.Compat.Base (Functor(..), Applicative(..), Monad(..))
import Basement.Monad
import Foundation.Monad.MonadIO
import Foundation.Monad.Exception
import Foundation.Monad.Transformer
import Foundation.Monad.Identity
