-- |
-- Module      : Foundation.List.ListN
-- License     : BSD-style
-- Maintainer  : Vincent Hanquez <vincent@snarc.org>
-- Stability   : experimental
-- Portability : portable
--
-- A Nat-sized list abstraction
--
-- Using this module is limited to GHC 7.10 and above.
--
module Foundation.List.ListN ( module X ) where

import Basement.Sized.List as X
